package server

import (
	"context"
	"time"

	def "git.pbiernat.dev/egommerce/api-entities/http"
	"git.pbiernat.dev/egommerce/basket-service/internal/app/service"
	"git.pbiernat.dev/egommerce/basket-service/internal/app/ui"
	"github.com/gofiber/fiber/v2"
)

func (s *Server) GetBasketHandler(c *fiber.Ctx) error {
	req := new(def.GetBasketRequest)
	if err := c.BodyParser(req); err != nil {
		return s.Error400(c, err.Error())
	}

	basketID := req.BasketID
	basketSrv := service.NewBasketService(s.db, s.cache, s.ebCh, s.log)
	ctx := context.Background()
	basket, err := basketSrv.FetchFromDB(ctx, basketID)
	if err != nil {
		return s.Error400(c, "Failed to retrieve basket")
	}

	res := &def.GetBasketResponse{
		ID:        basket.ID,
		State:     basket.State,
		CreatedAt: time.Duration(basket.CreatedAt.Time.Unix()),
	}
	if basket.UpdatedAt.Time.Unix() > 0 {
		res.UpdatedAt = time.Duration(basket.UpdatedAt.Time.Unix())
	}

	return c.JSON(res)
}

func (s *Server) GetBasketItemsHandler(c *fiber.Ctx) error {
	basketID := c.Params("basketId", "")
	basketSrv := service.NewBasketService(s.db, s.cache, s.ebCh, s.log)
	ctx := context.Background()
	items, err := basketSrv.FetchItems(ctx, basketID)
	if err != nil {
		return s.Error400(c, "Failed to retrieve basket items")
	}

	var res []*def.GetBasketItemsResponse // FIXME
	for _, item := range items {
		resItem := &def.GetBasketItemsResponse{
			ID:        item.ID,
			BasketID:  item.BasketID,
			ProductID: item.ProductID,
			Quantity:  item.Quantity,
			Price:     item.Price,
			CreatedAt: time.Duration(item.CreatedAt.Time.Unix()),
		}
		if item.UpdatedAt.Time.Unix() > 0 {
			resItem.UpdatedAt = time.Duration(item.UpdatedAt.Time.Unix())
		}
		res = append(res, resItem)
	}

	return c.JSON(res)
}

func (s *Server) CheckoutHandler(c *fiber.Ctx) error {
	reqID, _ := s.GetRequestID(c)
	req := new(def.BasketCheckoutRequest)
	if err := c.BodyParser(req); err != nil {
		return s.Error400(c, err.Error())
	}

	basketID := req.BasketID
	basketSrv := service.NewBasketService(s.db, s.cache, s.ebCh, s.log)
	res, err := ui.CheckoutBasket(basketSrv, basketID, reqID)
	if err != nil {
		return s.Error400(c, "Failed to create order")
	}

	return c.JSON(res)
}
